/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * Copyright (C) 2003 Imendio AB
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, see <https://www.gnu.org/licenses>
 */

#ifndef __LOUDMOUTH_H__
#define __LOUDMOUTH_H__

#define LM_INSIDE_LOUDMOUTH_H 1

#include <loudmouth/lm-connection.h>
#include <loudmouth/lm-error.h>
#include <loudmouth/lm-message.h>
#include <loudmouth/lm-message-handler.h>
#include <loudmouth/lm-message-node.h>
#include <loudmouth/lm-proxy.h>
#include <loudmouth/lm-utils.h>
#include <loudmouth/lm-ssl.h>

#undef LM_INSIDE_LOUDMOUTH_H

#endif /* __LOUDMOUTH_H__ */
